use egui::{Sense, Stroke, Vec2};

use super::{song_list::SongList, CompGetter, CompUi};


#[derive(Debug)]
pub struct Player {
    slider_progress: usize,
    old_slider_progress: usize,
    volume_slider: f64,
}

impl Default for Player {
    fn default() -> Self {
        Self {
            volume_slider: 1.0,
            old_slider_progress: 0,
            slider_progress: 0
        }
    }
}

component_register!(Player);


impl CompUi for Player {
    fn draw(ui: &mut egui::Ui, state: &mut crate::GuiState) -> crate::Result<()> {
        let theme = xmpd_settings::Settings::get()?.theme.clone();
        let avail = ui.available_size();
        ui.vertical_centered_justified(|ui| {
            ui.add_space(3.0);
            ui.horizontal(|ui| {
                {
                    ui.add_space(avail.x * 0.05 / 2.0);
                    ui.style_mut().spacing.slider_width = avail.x * 0.87;
                    let s = Stroke {
                        color: theme.accent_color,
                        width: 2.0
                    };
                    ui.style_mut().visuals.widgets.inactive.fg_stroke = s;
                    ui.style_mut().visuals.widgets.active.fg_stroke = s;
                    ui.style_mut().visuals.widgets.hovered.fg_stroke = s;

                    let mut slf = handle_error_ui!(Player::get());
                    ui.add(
                        egui::Slider::new(&mut slf.slider_progress, 0..=100)
                            .show_value(false)
                    );
                    if slf.slider_progress == slf.old_slider_progress {
                        slf.slider_progress = (state.player.get_played_f() * 100.0) as usize;
                        slf.old_slider_progress = slf.slider_progress;
                    } else {
                        handle_error_ui!(state.player.seek_to_f(slf.slider_progress as f64 / 100.0 ));
                        slf.old_slider_progress = slf.slider_progress;
                    }
                    let secs_left = state.player.get_ms_left() as f64 / 1000.0;
                    let h = (secs_left/60.0/60.0).floor();
                    let m = ((secs_left - h * 60.0)/60.0).floor();
                    let s = (secs_left - m * 60.0).floor();

                    ui.label(format!("{h:02}:{m:02}:{s:02}"));
                }
            });
            ui.horizontal(|ui| {
                ui.add_space((avail.x / 2.0) - 16.0 - 8.0 - ui.spacing().item_spacing.x);
                let pp = if state.player.is_paused() {
                    crate::data::PLAY_ICON
                } else {
                    crate::data::PAUSE_ICON
                };

                let prev = egui::Image::new(crate::data::PREV_ICON)
                    .tint(theme.accent_color)
                    .sense(Sense::click())
                    .max_size(Vec2::new(16.0, 16.0));
                let pp = egui::Image::new(pp)
                    .tint(theme.accent_color)
                    .sense(Sense::click())
                    .max_size(Vec2::new(16.0, 16.0));
                let next = egui::Image::new(crate::data::NEXT_ICON)
                    .tint(theme.accent_color)
                    .sense(Sense::click())
                    .max_size(Vec2::new(16.0, 16.0));
                if ui.add(prev).clicked() {     
                    handle_error_ui!(handle_error_ui!(SongList::get()).play_prev(state));
                }
                if ui.add(pp).clicked() {
                    if state.player.is_paused() {
                        state.player.play();
                    } else {
                        state.player.pause();
                    }
                }
                if ui.add(next).clicked() || state.player.just_stopped() {
                    handle_error_ui!(handle_error_ui!(SongList::get()).play_next(state));
                }



                ui.with_layout(egui::Layout::right_to_left(egui::Align::RIGHT), |ui| {
                    ui.add_space(15.0);
                    ui.style_mut().spacing.slider_width = avail.x * 0.15;
                    let s = Stroke {
                        color: theme.accent_color,
                        width: 1.0
                    };
                    ui.style_mut().visuals.widgets.inactive.fg_stroke = s;
                    ui.style_mut().visuals.widgets.active.fg_stroke = s;
                    ui.style_mut().visuals.widgets.hovered.fg_stroke = s;

                    let mut slf = handle_error_ui!(Player::get());
                    let slider =ui.add(
                        egui::Slider::new(&mut slf.volume_slider, 0.0..=1.0)
                            .show_value(false)
                    );

                    if slider.changed() {
                        state.player.set_volume(slf.volume_slider); 
                    }
                });
            });
            ui.add_space(3.0);
        });
        Ok(())
    }
}
