
macro_rules! component_register {
    ($comp:ident) => {
        lazy_static::lazy_static! {
            static ref __COMPONENT: std::sync::Arc<std::sync::Mutex<$comp>> = 
                std::sync::Arc::new(std::sync::Mutex::new($comp::default()));
        }
        impl crate::components::CompGetter for $comp {
            fn get() -> crate::Result<std::sync::MutexGuard<'static, Self>> {
                match __COMPONENT.lock() {
                    Ok(l) => Ok(l),
                    Err(e) => Err(anyhow::anyhow!(format!("{e:?}"))),
                }
            }
        }
    };
}

macro_rules! handle_error_ui {
    ($val:expr) => {
        match $val {
            Ok(v) => v,
            Err(e) => {
                use crate::components::CompGetter;
                log::error!("Error in {}:{}: {e}", std::file!(), std::line!());
                if let Ok(mut toast) = crate::components::toast::Toast::get() {
                    toast.show_toast(
                        &format!("Error in {}:{}", std::file!(), std::line!()),
                        &format!("{e}"),
                        crate::components::toast::ToastType::Error,
                    );
                }
                return;
            }
        } 
    };
}

macro_rules! handle_option {
    ($reason:expr, $val:expr) => {
        if let Some(v) = $val {
            v
        } else {
            handle_error_ui!(Err(anyhow::anyhow!($reason)));
            return;
        }
    };
}
